package com.city.cdcmonitor.http;

import com.city.cdcmonitor.exception.CityException;
import com.city.cdcmonitor.utils.ValidateUtils;
import com.google.gson.Gson;
import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.Map;

public class HttpSender {
    private static Logger logger = LoggerFactory.getLogger(HttpSender.class);

    public static String post(String url, Object params) {
        return post(url, params, null);
    }

    public static String post(String url, Object params, Map<String, String> headers) {
        return sendPost(url, new Gson().toJson(params), headers);
    }

    private static String sendPost(String url, String params, Map<String, String> headers) {
        String result = null;
        //创建post请求方法实例对象
        PostMethod postMethod = new PostMethod(url);
        logger.info(" ===>>> send post request: " + params);
        try {
            //创建httpClient实例对象
            HttpClient client = new HttpClient();
            //设置httpClient连接主机服务器超时时间
            client.getHttpConnectionManager().getParams().setConnectionTimeout(5000);
            //charset
            client.getParams().setContentCharset("UTF-8");
            //设置post请求超时时间
            postMethod.getParams().setParameter(HttpMethodParams.SO_TIMEOUT, 5000);
            postMethod.addRequestHeader("Accept", "application/json; charset=UTF-8");
            postMethod.addRequestHeader("Content-Type", "application/json; charset=UTF-8");
            //自定义请求头
            if (ValidateUtils.isNotEmpty(headers)) {
                for (Map.Entry<String, String> entry : headers.entrySet()) {
                    postMethod.addRequestHeader(entry.getKey(), entry.getValue());
                }
            }

            // 设置为默认的重试策略
            postMethod.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, new DefaultHttpMethodRetryHandler());
            //请求体
            StringRequestEntity requestEntity = new StringRequestEntity(params, "application/json", "UTF-8");
            postMethod.setRequestEntity(requestEntity);
            //执行请求
            int i = client.executeMethod(postMethod);
            logger.info(" ===>>> send post response code: " + i);
            //获取返回信息
            result = postMethod.getResponseBodyAsString();
            logger.info(" ===>>> send post response: " + result);
        } catch (IOException e) {
            logger.error("发送POST application/json请求出现异常！" + e);
            throw new RuntimeException(e);
        } finally {
            postMethod.releaseConnection();
        }
        return result;
    }

    public static String get(String url, Map<String, String> params) {
        return get(url, params, null);
    }

    public static String get(String url, Map<String, String> params, Map<String, String> headers) {
        return sendGet(url, getUrlParamsFromMap(params), headers);
    }

    private static String sendGet(String url, String param, Map<String, String> headers) {
        StringBuilder result = new StringBuilder();
        InputStream inputStream = null;
        InputStreamReader inputStreamReader = null;
        BufferedReader bufferedReader = null;
        try {
            String urlNameString = url + "?" + param;
            logger.info(" ===>>> send get request: " + urlNameString);
            URL realUrl = new URL(urlNameString);
            //打开和URL之间的连接
            URLConnection connection = realUrl.openConnection();
            //设置通用的请求属性
            connection.setRequestProperty("Content-Length", "0");
            connection.setRequestProperty("Host", "<calculated when request is sent>");
            connection.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            connection.setRequestProperty("Accept", "*/*");
            connection.setRequestProperty("Accept-Encoding", "gzip, deflate, br");
            connection.setRequestProperty("connection", "keep-alive");
            //自定义请求头
            if (ValidateUtils.isNotEmpty(headers)) {
                for (Map.Entry<String, String> entry : headers.entrySet()) {
                    connection.setRequestProperty(entry.getKey(), entry.getValue());
                }
            }
            //建立实际的连接
            connection.connect();
            //定义BufferedReader输入流来读取URL的响应
            inputStream = connection.getInputStream();
            inputStreamReader = new InputStreamReader(inputStream, "UTF-8");
            bufferedReader = new BufferedReader(inputStreamReader);
            String line;
            while ((line = bufferedReader.readLine()) != null) {
                result.append(line);
            }
            logger.info(" ===>>> send get response: " + result.toString());
        } catch (Exception e) {
            logger.error("发送GET请求出现异常！" + e);
            throw new RuntimeException(e);
        } finally {
            try {
                if (bufferedReader != null) {
                    bufferedReader.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

            try {
                if (inputStreamReader != null) {
                    inputStreamReader.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

            try {
                if (inputStream != null) {
                    inputStream.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return result.toString();
    }

    private static String getUrlParamsFromMap(Map<String, String> params) {
        try {
            if (ValidateUtils.isNotEmpty(params)) {
                StringBuilder stringBuilder = new StringBuilder();
                for (Map.Entry<String, String> entry : params.entrySet()) {
                    stringBuilder.append(URLEncoder.encode(entry.getKey(), "UTF-8")).append("=").append(URLEncoder.encode(entry.getValue(), "UTF-8")).append("&");
                }
                String content = stringBuilder.toString();
                if (content.endsWith("&")) {
                    int pos = content.lastIndexOf("&");
                    if (pos > -1) {
                        content = content.substring(0, pos);
                    }
                }
                return content;
            }
        } catch (Exception e) {
            CityException.error(e);
        }
        return "";
    }

    /**
     * 测试接口是否能通
     * @param url
     * @return
     */
    public static int getResponseCode(String url) {
        try {
            logger.info(" ===>>> send get request: " + url);
            URL realUrl = new URL(url);
            //打开和URL之间的连接
            HttpURLConnection connection = (HttpURLConnection) realUrl.openConnection();
            connection.setRequestMethod("GET");
            connection.setConnectTimeout(3000);
            connection.setReadTimeout(3000);
            //设置通用的请求属性

            //建立实际的连接
            return connection.getResponseCode();
        } catch (Exception e) {
            logger.error("发送GET请求出现异常！" + e);
            return 404;
        }
    }
}
