package com.city.cdcmonitor.service;

import com.city.cdcmonitor.bean.FileSpaceBean;
import com.city.cdcmonitor.bean.InterfaceHealthBean;
import com.city.cdcmonitor.http.HttpSender;
import com.city.cdcmonitor.utils.PropertyConfig;
import com.city.cdcmonitor.utils.ValidateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.curator.RetryPolicy;
import org.apache.curator.framework.CuratorFramework;
import org.apache.curator.framework.CuratorFrameworkFactory;
import org.apache.curator.retry.ExponentialBackoffRetry;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class CheckHealthService {
    @Value("${error.mail}")
    private String errorMail;
    @Value("${spring.kafka.city.180.ip}")
    private String ip180;
    @Value("${spring.kafka.city.188.ip}")
    private String ip188;
    @Resource
    private KafkaHeartbeatService kafkaHeartbeatService;

    /**
     * 从 external-interface-health.properties 配置文件中获取调用地址
     * 发送请求, 失败发邮件
     */
    public List<InterfaceHealthBean> externalInterfaceHealth() {
        List<InterfaceHealthBean> tableList = new ArrayList<>();
        //配置文件地址调用
        Map<String, String> ips = PropertyConfig.getExternalInterfaceHealthIp();
        for (String key : ips.keySet()) {
            String ip = ips.get(key);
            InterfaceHealthBean healthBean = new InterfaceHealthBean();
            healthBean.setInterfaceName(ip);
            try {
                int responseCode = HttpSender.getResponseCode(ip);
                if (responseCode == 404) {
                    healthBean.setStatus("404");
                } else {
                    healthBean.setStatus("正常");
                }
            } catch (Exception e) {
                healthBean.setStatus("发送请求失败");
            }
            tableList.add(healthBean);
        }
        //180
//        tableList.add(zookeeperHealth(ip180));
        tableList.add(kafkaHealth(ip180));
        //180 send city kafka
        tableList.add(cityKafkaHealth(ip180));
        //180 send cdc kafka
        tableList.add(cdcKafkaHealth(ip180));

        //188
//        tableList.add(zookeeperHealth(ip188));
        tableList.add(kafkaHealth(ip188));
        //180 send city kafka
        tableList.add(cityKafkaHealth(ip188));
        //188 send cdc kafka
        tableList.add(cdcKafkaHealth(ip188));

        System.out.println(tableList);
        return tableList;
    }

    public InterfaceHealthBean zookeeperHealth(String ip) {
        String connectString = ip + ":2181";//地址和端口，集群用,隔开
        //重试策略, 等1秒, 重试1次
        RetryPolicy retryPolicy = new ExponentialBackoffRetry(1000, 1);
        //zookeeper客户端
        CuratorFramework client = CuratorFrameworkFactory.builder()
                .connectString(connectString)
                .sessionTimeoutMs(1000)
                .connectionTimeoutMs(1000)
                .retryPolicy(retryPolicy)
                .namespace("app")  //设置一个根目录, 所有操作都在app下
                .build();

        InterfaceHealthBean healthBean = new InterfaceHealthBean();
        healthBean.setInterfaceName("zookeeper." + ip);
        try {
            client.start();
            client.getData().forPath("/");
            healthBean.setStatus("正常");
        } catch (Exception e) {
            log.error(e.getMessage());
            healthBean.setStatus("连接失败");
        }

        return healthBean;
    }

    public InterfaceHealthBean kafkaHealth(String ip) {
        InterfaceHealthBean healthBean = new InterfaceHealthBean();
        healthBean.setInterfaceName("kafka." + ip);
        try {
            kafkaHeartbeatService.heartbeat(ip + ":9000", 30000);
            healthBean.setStatus("正常");
        } catch (Exception e) {
            healthBean.setStatus("生产消息失败");
        }
        return healthBean;
    }

    public InterfaceHealthBean cdcKafkaHealth(String ip) {
        String ipName = ip.endsWith("180") ? "180" : "188";
        InterfaceHealthBean sendCdcKafka = new InterfaceHealthBean();
        sendCdcKafka.setInterfaceName(ipName + "触发cdc kafka:6404");
        try {
            String result180 = HttpSender.post("http://" + ip + ":8080/heartbeatCdc", null);
            sendCdcKafka.setStatus("success".equals(result180) ? "正常" : "生产消息失败");
        } catch (Exception e) {
            sendCdcKafka.setStatus("404");
        }
        return sendCdcKafka;
    }

    public InterfaceHealthBean cityKafkaHealth(String ip) {
        String ipName = ip.endsWith("180") ? "180" : "188";
        InterfaceHealthBean sendCdcKafka = new InterfaceHealthBean();
        sendCdcKafka.setInterfaceName(ipName + "触发本机 kafka:9000");
        try {
            String result180 = HttpSender.post("http://" + ip + ":8080/heartbeatCity", null);
            sendCdcKafka.setStatus("success".equals(result180) ? "正常" : "生产消息失败");
        } catch (Exception e) {
            sendCdcKafka.setStatus("404");
        }
        return sendCdcKafka;
    }

    private String[] checkFileLengthPath = new String[]{"/citygis/kafka", "/opt/kafka-3.2.1/logs", "/home/citygis/cdc_kafka/logs"};

    /**
     * 服务器文件大小监听
     */
    public List<FileSpaceBean> fileLengthHealth() {
        List<FileSpaceBean> tableList = new ArrayList<>();
        //180
        for (int i = 0; i < checkFileLengthPath.length; i++) {
            String fileName = checkFileLengthPath[i];
            Map<String, String> params = new HashMap<>();
            params.put("filePath", fileName);
            FileSpaceBean file180 = new FileSpaceBean();
            try {
                String result180 = HttpSender.get("http://172.27.128.180:8080/getFilesLength", params);
                file180.setFilePath("180-" + fileName);
                if (ValidateUtils.isNotEmpty(result180) && result180.contains("-")) {
                    String[] split = result180.split("-");
                    file180.setFileSpace(split[0]);
                    file180.setUsableSpace(split[1]);
                    file180.setFileTotalSpace(split[2]);
                    tableList.add(file180);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        //188
        for (int i = 0; i < checkFileLengthPath.length; i++) {
            String fileName = checkFileLengthPath[i];
            Map<String, String> params = new HashMap<>();
            params.put("filePath", fileName);
            FileSpaceBean file188 = new FileSpaceBean();
            try {
                String result188 = HttpSender.get("http://172.27.128.188:8080/getFilesLength", params);
                file188.setFilePath("188-" + fileName);
                if (ValidateUtils.isNotEmpty(result188) && result188.contains("-")) {
                    String[] split = result188.split("-");
                    file188.setFileSpace(split[0]);
                    file188.setUsableSpace(split[1]);
                    file188.setFileTotalSpace(split[2]);
                }
                tableList.add(file188);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        System.out.println(tableList);
        return tableList;
    }


}
