package com.city.cdcmonitor.service;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.annotation.format.DateTimeFormat;
import com.city.cdcmonitor.bean.ExportColumnBean;
import com.city.cdcmonitor.exception.CityException;
import com.city.cdcmonitor.utils.DateUtils;
import com.city.cdcmonitor.utils.EntityUtils;
import com.city.cdcmonitor.utils.ValidateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Description;
import org.springframework.core.io.FileSystemResource;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.mail.internet.InternetAddress;
import java.io.File;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.*;

@Slf4j
@Service
public class SendEmailService {
    @Value("${spring.mail.username}")
    private String sender;
    @Value("${copy.mail}")
    private String copyMail;
    @Value("${temp.excel.filePath}")
    private String tempFilePath;
    @Resource
    private JavaMailSender mailSender;

    /**
     * 发送email方法
     *
     * @param subject:  标题
     * @param text:     内容
     * @param files:    附件
     * @param copyFlag: 是否抄送
     * @param to:       收件人
     * @return
     */
    public Map<String, Object> sendMail(String subject, String text, List<Map<String, Object>> files, boolean copyFlag, String... to) {
        Map<String, Object> result = new HashMap<>(16);
        mailSender.send(message -> {
            MimeMessageHelper mmh = new MimeMessageHelper(message, true);
            // 设置自定义发件人昵称
            String nick = "";
            try {
                nick = javax.mail.internet.MimeUtility.encodeText("六统一");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            InternetAddress mail = new InternetAddress(nick + " <" + sender + ">");
            mmh.setFrom(mail);
            //收件人
            mmh.setTo(to);
            //抄送
            if (copyFlag) {
                mmh.setCc(copyMail.split(","));
            }
            //主题
            mmh.setSubject(subject);
            //内容
            mmh.setText(text, true);
            //附件
            for (int i = 0; ValidateUtils.isNotEmpty(files) && i < files.size(); i++) {
                String filePath = (String) files.get(i).get("filePath");
                String fileName = (String) files.get(i).get("fileName");
                String absolutePath = null;
                if (filePath.endsWith(fileName)) {
                    absolutePath = filePath;
                } else {
                    absolutePath = filePath + fileName;
                }
                File file = new File(absolutePath);
                if (!file.exists()) {
                    continue;
                }
                FileSystemResource fileSystemResource = new FileSystemResource(file);
                mmh.addAttachment(fileName, fileSystemResource);
            }
        });

        result.put("respCode", "0000");
        result.put("respDesc", "邮件发送成功");
        return result;
    }

    public Map<String, Object> sendMail(String subject, String text, boolean copyFlag, String... to) {
        return this.sendMail(subject, text, null, copyFlag, to);
    }

    /**
     * 多个table拼接
     *
     * @param subject:  标题
     * @param text:     table的html
     * @param copyFlag: 是否抄送
     * @param to:       收件人
     * @return
     */
    public Map<String, Object> sendMailAsTable(String subject, String text, boolean copyFlag, String... to) {
        StringBuilder sb = new StringBuilder();
        //创建邮件html
        sb.append("<html><head><style>");
        sb.append("table {}");
        sb.append("td {border-right: 1px solid #1E90FF;border-bottom: 1px solid #1E90FF;}");
        sb.append("th {background: #95d475;}");
        sb.append("td {background: #F5FAFA;}");
        sb.append("</style></head><body>");
        //实际内容
        sb.append(text);
        //html结尾
        sb.append("</body></html>");
        return this.sendMail(subject, sb.toString(), copyFlag, to);
    }

    /**
     * 邮件内容写入table
     *
     * @param title:    数据标题
     * @param dataList: 表格数据
     * @return
     */
    public String getHtmlTable(String title, List<?> dataList) {
        //表格数据
        StringBuilder sb = new StringBuilder();
        if (ValidateUtils.isEmpty(dataList)) {
            sb.append("<h3>无记录</h3>");
        } else {
            List<ExportColumnBean> columnList = EntityUtils.getExportColumnList(dataList.get(0).getClass());
            sb.append("<h3>").append(title).append(": </h3>");
            //创建表头
            sb.append("<table><tr>");
            for (ExportColumnBean columnBean : columnList) {
                sb.append("<th align='center'>").append(columnBean.getDescription()).append("</td>");
            }
            sb.append("</tr>");
            //创建标题
            for (Object obj : dataList) {
                sb.append("<tr>");
                try {
                    Class<?> clazz = obj.getClass();
                    //给字段名字首字母大写, 并加上get前缀, 反射调用get方法
                    for (ExportColumnBean columnBean : columnList) {
                        //获取getter方法名
                        String getter = "get" + StringUtils.capitalize(columnBean.getColumnName());
                        //获取getter方法
                        Method m = clazz.getDeclaredMethod(getter);
                        //临时改变访问权限
                        m.setAccessible(true);
                        Object value = m.invoke(obj);
                        Field field = clazz.getDeclaredField(columnBean.getColumnName());
                        if (field.isAnnotationPresent(DateTimeFormat.class)) {
                            //获取字段的@DateTimeFormat值
                            DateTimeFormat declaredAnnotation = field.getDeclaredAnnotation(DateTimeFormat.class);
                            String pattern = declaredAnnotation.value();
                            SimpleDateFormat sdf = new SimpleDateFormat(pattern);
                            value = sdf.format(value);
                        }
                        sb.append("<td>").append(value).append("</td>");
                    }
                } catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException | NoSuchFieldException e) {
                    CityException.error(e);
                }
                sb.append("</tr>");
            }
            sb.append("</table><br/><br/>");
        }
        return sb.toString();
    }

    /**
     * 邮件内容写入table
     *
     * @param dataMap: 表格数据
     * @return
     */
    public String getHtmlTable(LinkedHashMap<String, List<LinkedHashMap<String, Object>>> dataMap) {
        //表格数据
        StringBuilder sb = new StringBuilder();
        if (ValidateUtils.isEmpty(dataMap)) {
            sb.append("<h3>无记录</h3><br/><br/>");
        } else {
            for (String key : dataMap.keySet()) {
                //创建<table>
                sb.append("<h3>").append(key).append(": </h3>");
                List<LinkedHashMap<String, Object>> resultList = dataMap.get(key);
                if (ValidateUtils.isEmpty(resultList)) {
                    sb.append("<h3>无记录</h3><br/><br/>");
                } else {
                    sb.append("<table>");
                    List<ExportColumnBean> columnList = null;
                    if ("view_日报(当日)".equals(key.trim())) {
                        columnList = EntityUtils.getExportColumnList4();
                    } else if ("view_日报(昨日)".equals(key.trim())) {
                        columnList = EntityUtils.getExportColumnList5();
                    } else {
                        columnList = EntityUtils.getExportColumnList(resultList.get(0));
                    }
                    //创建表头
                    sb.append("<tr>");
                    for (ExportColumnBean columnBean : columnList) {
                        sb.append("<th align='center'>").append(columnBean.getDescription()).append("</td>");
                    }
                    sb.append("</tr>");
                    //创建标题
                    for (LinkedHashMap<String, Object> data : resultList) {
                        sb.append("<tr>");
                        for (ExportColumnBean columnBean : columnList) {
                            Object value = data.get(columnBean.getColumnName());
                            if (value != null) {
                                if (value instanceof java.sql.Date) {
                                    value = new SimpleDateFormat("yyyy-MM-dd").format(value);
                                } else if (value instanceof java.sql.Time) {
                                    value = new SimpleDateFormat("HH:mm:ss").format(value);
                                } else if (value instanceof java.sql.Timestamp) {
                                    value = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(value);
                                }
                            }
                            sb.append("<td>");
                            //特殊要求, 添加颜色
                            boolean redFlag = false;
                            if ("数量对比差值".equals(columnBean.getColumnName().trim())) {
                                if (ValidateUtils.isInteger(value)) {
                                    int count = Integer.parseInt(value.toString());
                                    if (count != 0) {
                                        redFlag = true;
                                    }
                                }
                            }
                            if (value == null || "null".equals(value)) {
                                value = "";
                            }
                            if (redFlag) {
                                sb.append("<span style=\"color: red;\">").append(value).append("</span>");
                            } else {
                                sb.append(value);
                            }
                            sb.append("</td>");
                        }
                        sb.append("</tr>");
                    }
                    sb.append("</table><br/><br/>");
                }
            }
        }
        return sb.toString();
    }
}
