package com.city.cdcmonitor.utils;

import lombok.extern.slf4j.Slf4j;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

@Slf4j
public class DateUtils {
    public static final String DATE_PATTERN1 = "yyyyMMdd";
    public static final String DATE_PATTERN2 = "yyyy-MM-dd";
    public static final String DATE_PATTERN3 = "yyyyMMddHHmmss";
    public static final String DATE_PATTERN4 = "yyyy-MM-dd HH:mm:ss";
    public static final String DATE_PATTERN5 = "yyyy-MM-dd'T'HH:mm:ss'Z'";
    public static final String DATE_PATTERN6 = "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'";
    public static final String DATE_PATTERN7 = "yyyyMM";
    public static final String DATE_PATTERN8 = "yyyy-MM";

    /**
     * 获取 yyyyMMddHHmmss 字符串
     *
     * @return
     */
    public static String getCurrentTime() {
        return dateToString(new Date(), DATE_PATTERN3);
    }

    /**
     * 获取 yyyy-MM-dd 字符串
     */
    public static String dateToStringYYYYMMDD(Date date) {
        return dateToString(date, DATE_PATTERN2);
    }

    /**
     * 获取 yyyy-MM-dd HH:mm:ss 字符串
     */
    public static String dateToStringYYYYMMDDHHMMSS(Date date) {
        return dateToString(date, DATE_PATTERN4);
    }

    /**
     * 获取指定格式的字符串
     */
    public static String dateToString(Date date, String pattern) {
        if (date == null) {
            throw new IllegalArgumentException("date is null");
        }
        if (ValidateUtils.isEmpty(pattern)) {
            throw new IllegalArgumentException("pattern is null");
        }
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        return sdf.format(date);
    }

    /**
     * 获取 yyyy-MM-dd 字符串转化成的时间
     */
    public static Date stringToDateYYYYMMDD(String dateStr) {
        return stringToDate(dateStr, DATE_PATTERN2);
    }

    /**
     * 获取 yyyy-MM-dd HH:mm:ss 字符串转化成的时间
     */
    public static Date stringToDateYYYYMMDDHHMMSS(String dateStr) {
        return stringToDate(dateStr, DATE_PATTERN4);
    }

    /**
     * 获取指定格式的字符串转化成的时间
     */
    public static Date stringToDate(String dateStr, String pattern) {
        if (ValidateUtils.isEmpty(dateStr)) {
            throw new IllegalArgumentException("date is null");
        }
        if (ValidateUtils.isEmpty(pattern)) {
            throw new IllegalArgumentException("pattern is null");
        }
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        Date date = null;
        try {
            date = sdf.parse(dateStr);
        } catch (ParseException e) {
            log.error("格式化日期异常");
        }
        return date;
    }

    /**
     * 计算两个日期之间相差的天数
     *
     * @param smallerDate: 较小的时间
     * @param biggerDate: 较大的时间
     * @return 相差天数
     */
    public static int daysBetween(Date smallerDate, Date biggerDate) {
        smallerDate = getDateRemoveTime(smallerDate);
        biggerDate = getDateRemoveTime(biggerDate);
        Calendar cal = Calendar.getInstance();
        cal.setTime(smallerDate);
        long time1 = cal.getTimeInMillis();
        cal.setTime(biggerDate);
        long time2 = cal.getTimeInMillis();
        long between_days = (time2 - time1) / (1000 * 3600 * 24);
        return Integer.parseInt(String.valueOf(between_days));
    }

    /**
     * 日期，去除时分秒
     */
    public static Date getDateRemoveTime(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTime();
    }

    /**
     * 判断夜间1，白天0 夜间：0时 < T < 6时； 白天： 6时 <= T <= 24时
     */
    public static boolean isNight(Date date) {
        SimpleDateFormat df = new SimpleDateFormat("HH:mm");
        Date now = null;
        Date beginTime = null;
        Date endTime = null;
        try {
            now = df.parse(df.format(date));
            beginTime = df.parse("00:00");
            endTime = df.parse("06:00");
            return belongCalendar(now, beginTime, endTime);
        } catch (Exception e) {
            e.getStackTrace();
        }
        return false;

    }

    /**
     * 判断时间是否在时间段内
     * @param nowTime
     * @param beginTime
     * @param endTime 结束时间当天不算，如要计算 结束时间+1天
     * @return
     */
    public static boolean belongCalendar(Date nowTime, Date beginTime, Date endTime) {
        Calendar date = Calendar.getInstance();
        date.setTime(nowTime);

        Calendar begin = Calendar.getInstance();
        begin.setTime(beginTime);

        Calendar end = Calendar.getInstance();
        end.setTime(endTime);

        return date.after(begin) && date.before(end);
    }

    /**
     * 获取上个星期的某一天
     * @param dateStr
     * @param pattern
     * @param field: Calendar.SUNDAY ~ Calendar.SATURDAY, 1~7
     * @return
     */
    public static String getLastWeekDay(String dateStr, String pattern, int field) {
        if (ValidateUtils.isEmpty(dateStr)) {
            throw new IllegalArgumentException("date is null");
        }
        if (ValidateUtils.isEmpty(pattern)) {
            throw new IllegalArgumentException("pattern is null");
        }
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        Date date = null;
        try {
            date = sdf.parse(dateStr);
        } catch (ParseException e) {
            log.error("格式化日期异常");
            throw new IllegalArgumentException("date parse is null");
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        if (field != Calendar.SUNDAY) {
            //据《圣经》故事中记载，上帝用6天创造世界万物，在第7天休息，这7天是从星期日开始的，第7天是星期六
            calendar.add(Calendar.DATE, -7);
        }
        calendar.set(Calendar.DAY_OF_WEEK, field);
        return sdf.format(calendar.getTime());
    }
}
